@@ -4,24 +4,14 @@
  * Copyright (C) 1997-2002 Marko Macek
  */
 #include "config.h"
-
-#include "yfull.h"
-#include "ykey.h"
 #include "wmframe.h"
-
-#include "wmclient.h"
 #include "wmstatus.h"
-#include "yxapp.h"
 #include "wmapp.h"
-#include "yrect.h"
 #include "prefs.h"
-
+#include "wmtaskbar.h"
 #include "intl.h"
 
-#include "wmtaskbar.h"
-#include "aworkspaces.h"
-
-#include <stdio.h>
+extern YColorName activeBorderBg;
 
 void YFrameWindow::snapTo(int &wx, int &wy,
                           int rx1, int ry1, int rx2, int ry2,
@@ -179,30 +169,38 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
     wy = yp;
 }
 
-void YFrameWindow::drawMoveSizeFX(int x, int y, int w, int h, bool) {
+void YFrameWindow::drawMoveSizeFX(int x, int y, int w, int h) {
     if ((movingWindow && opaqueMove) ||
         (sizingWindow && opaqueResize))
         return;
 
-    int const bw((wsBorderX + wsBorderY) / 2);
-    int const bo((wsBorderX + wsBorderY) / 4);
-    static Graphics * gc(NULL);
+    int const pencil(min(3U, (wsBorderX + wsBorderY) / 2));
+    int const offset(min(2U, (wsBorderX + wsBorderY) / 4));
 
-    if (gc == NULL) {
-        XGCValues gcv;
-
-        gcv.foreground = YColor(clrActiveBorder).pixel();
-        gcv.function = GXxor;
-        gcv.graphics_exposures = False;
-        gcv.line_width = bw;
-        gcv.subwindow_mode = IncludeInferiors;
-
-        gc = new Graphics(*desktop, GCForeground | GCFunction |
-                                    GCGraphicsExposures | GCLineWidth |
-                                    GCSubwindowMode, &gcv);
+    YColor color(activeBorderBg);
+    unsigned mask = color.red() | color.green() | color.blue();
+    if (mask < 32) {
+        const int gray = 63;
+        color = YColor(gray, gray, gray);
+    }
+    else if (mask < 64) {
+        const int k = 2;
+        color = YColor(k * color.red(), k * color.green(), k * color.blue());
+    }
+    else if (mask < 128) {
+        color = color.brighter().brighter();
     }
 
-    gc->drawRect(x + bo, y + bo, w - bw, h - bw);
+    XGCValues gcv = { GXxor, };
+
+    gcv.foreground = color.pixel();
+    gcv.line_width = pencil;
+    gcv.subwindow_mode = IncludeInferiors;
+    gcv.graphics_exposures = False;
+
+    Graphics g(*desktop, GCFunction | GCForeground | GCLineWidth |
+                         GCSubwindowMode | GCGraphicsExposures, &gcv);
+    g.drawRect(x + offset, y + offset, w - pencil, h - pencil);
 }
 
 int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
@@ -334,43 +332,42 @@ void YFrameWindow::handleMoveMouse(const XMotionEvent &motion, int &newX, int &n
     int mx, my, Mx, My;
     manager->getWorkArea(this, &mx, &my, &Mx, &My);
 
-
     if (!(motion.state & ShiftMask)) {
         if (/*EdgeResistance >= 0 && %%% */ EdgeResistance < 10000) {
-            if (newX + int(width() + n * borderX()) > Mx) {
-                if (newX + int(width() + n * borderX()) < int(Mx + EdgeResistance))
-                    newX = Mx - width() - n * borderX();
+            if (newX + int(width()) + n * borderX() > Mx) {
+                if (newX + int(width()) + n * borderX() < Mx + EdgeResistance)
+                    newX = Mx - int(width()) - n * borderX();
                 else if (motion.state & ShiftMask)
                     newX -= EdgeResistance;
             }
-            if (newY + int(height() + n * borderY()) > My) {
-                if (newY + int(height() + n * borderY()) < int(My + EdgeResistance))
-                    newY = My - height() - n * borderY();
+            if (newY + int(height()) + n * borderY() > My) {
+                if (newY + int(height()) + n * borderY() < My + EdgeResistance)
+                    newY = My - int(height()) - n * borderY();
                 else if (motion.state & ShiftMask)
                     newY -= EdgeResistance;
             }
             if (newX < mx) {
-                if (newX > int(- EdgeResistance + mx))
+                if (newX > mx - EdgeResistance)
                     newX = mx;
                 else if (motion.state & ShiftMask)
                     newX += EdgeResistance;
             }
             if (newY < my) {
-                if (newY > int(- EdgeResistance + my))
+                if (newY > my - EdgeResistance)
                     newY = my;
                 else if (motion.state & ShiftMask)
                     newY += EdgeResistance;
             }
         }
         if (EdgeResistance == 10000 || isMaximizedHoriz()) {
-            if (newX + int(width() + n * borderX()) > Mx)
-                newX = Mx - width() - n * borderX();
+            if (newX + int(width()) + n * borderX() > Mx)
+                newX = Mx - int(width()) - n * borderX();
             if (newX < mx)
                 newX = mx;
         }
         if (EdgeResistance == 10000 || isMaximizedVert()) {
-            if (newY + int(height() + n * borderY()) > My)
-                newY = My - height() - n * borderY();
+            if (newY + int(height()) + n * borderY() > My)
+                newY = My - int(height()) - n * borderY();
             if (newY < my)
                 newY = my;
         }
@@ -382,6 +379,9 @@ void YFrameWindow::handleMoveMouse(const XMotionEvent &motion, int &newX, int &n
 void YFrameWindow::handleResizeMouse(const XMotionEvent &motion,
                                      int &newX, int &newY, int &newWidth, int &newHeight)
 {
+    if (buttonDownX == 0 && buttonDownY == 0 && grabX == 0 && grabY == 0)
+        return;
+
     int mouseX = motion.x_root;
     int mouseY = motion.y_root;
 
@@ -415,27 +415,23 @@ void YFrameWindow::handleResizeMouse(const XMotionEvent &motion,
         newX = x() + width() - newWidth;
     if (grabY == -1)
         newY = y() + height() - newHeight;
-
 }
 
 void YFrameWindow::outlineMove() {
     int xx(x()), yy(y());
-    unsigned modifiers(0);
 
     XGrabServer(xapp->display());
-    XSync(xapp->display(), False);
 
-    for(;;) {
+    for (;;) {
         XEvent xev;
 
         XWindowEvent(xapp->display(), handle(),
-                     KeyPressMask | ExposureMask | 
-                     ButtonPressMask | ButtonReleaseMask | 
+                     KeyPressMask | ExposureMask |
+                     ButtonPressMask | ButtonReleaseMask |
                      PointerMotionMask, &xev);
 
         switch (xev.type) {
             case KeyPress: {
-                modifiers = xev.xkey.state;
 
                 int const ox(xx), oy(yy);
                 int r;
@@ -445,9 +441,7 @@ void YFrameWindow::outlineMove() {
                     case -2:
                         if (xx != ox || yy != oy) {
                             drawMoveSizeFX(ox, oy, width(), height());
-#ifndef LITE
                             statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
-#endif
                             drawMoveSizeFX(xx, yy, width(), height());
                         }
 
@@ -468,7 +462,6 @@ void YFrameWindow::outlineMove() {
 
             case ButtonPress:
             case ButtonRelease:
-                modifiers = xev.xbutton.state;
                 goto end;
 
             case MotionNotify: {
@@ -478,9 +471,7 @@ void YFrameWindow::outlineMove() {
 
                 if (xx != ox || yy != oy) {
                     drawMoveSizeFX(ox, oy, width(), height());
-#ifndef LITE
                     statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
-#endif
                     drawMoveSizeFX(xx, yy, width(), height());
                 }
 
@@ -507,14 +498,13 @@ void YFrameWindow::outlineResize() {
     }
 
     XGrabServer(xapp->display());
-    XSync(xapp->display(), False);
 
-    for(;;) {
+    for (;;) {
         XEvent xev;
 
         XWindowEvent(xapp->display(), handle(),
                      KeyPressMask | ExposureMask |
-                     ButtonPressMask | ButtonReleaseMask | 
+                     ButtonPressMask | ButtonReleaseMask |
                      PointerMotionMask, &xev);
 
         switch (xev.type) {
@@ -528,9 +518,7 @@ void YFrameWindow::outlineResize() {
                     case 1:
                         if (ox != xx || oy != yy || ow != ww || oh != hh) {
                             drawMoveSizeFX(ox, oy, ow, oh);
-#ifndef LITE
                             statusMoveSize->setStatus(this, YRect(xx, yy, ww, hh));
-#endif
                             drawMoveSizeFX(xx, yy, ww, hh);
                         }
 
@@ -545,7 +533,7 @@ void YFrameWindow::outlineResize() {
                     case -1:
                         goto end;
                 }
-                
+
                 break;
             }
 
@@ -560,9 +548,7 @@ void YFrameWindow::outlineResize() {
 
                 if (ox != xx || oy != yy || ow != ww || oh != hh) {
                     drawMoveSizeFX(ox, oy, ow, oh);
-#ifndef LITE
                     statusMoveSize->setStatus(this, YRect(xx, yy, ww, hh));
-#endif
                     drawMoveSizeFX(xx, yy, ww, hh);
                 }
 
@@ -593,20 +579,18 @@ void YFrameWindow::manualPlace() {
     buttonDownY = 0;
 
     if (!xapp->grabEvents(desktop,
-                          YXApplication::movePointer.handle(),
+                          YXApplication::movePointer,
                           ButtonPressMask |
                           ButtonReleaseMask |
                           PointerMotionMask))
         return;
 
     XGrabServer(xapp->display());
-#ifndef LITE
     statusMoveSize->begin(this);
-#endif
 
     drawMoveSizeFX(xx, yy, width(), height());
 
-    for(;;) {
+    for (;;) {
         XEvent xev;
 
         XMaskEvent(xapp->display(),
@@ -625,9 +609,7 @@ void YFrameWindow::manualPlace() {
                     case -2:
                         if (xx != ox || yy != oy) {
                             drawMoveSizeFX(ox, oy, width(), height());
-#ifndef LITE
                             statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
-#endif
                             drawMoveSizeFX(xx, yy, width(), height());
                         }
 
@@ -645,7 +627,7 @@ void YFrameWindow::manualPlace() {
 
                 break;
             }
-            
+
 
             case ButtonPress:
             case ButtonRelease:
@@ -657,9 +639,7 @@ void YFrameWindow::manualPlace() {
                 handleMoveMouse(xev.xmotion, xx, yy);
                 if (xx != ox || yy != oy) {
                     drawMoveSizeFX(ox, oy, width(), height());
-#ifndef LITE
                     statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
-#endif
                     drawMoveSizeFX(xx, yy, width(), height());
                 }
 
@@ -671,9 +651,7 @@ void YFrameWindow::manualPlace() {
 end:
     drawMoveSizeFX(xx, yy, width(), height());
 
-#ifndef LITE
     statusMoveSize->end();
-#endif
     moveWindow(xx, yy);
     xapp->releaseEvents();
     XUngrabServer(xapp->display());
@@ -688,7 +666,7 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
             switch (handleMoveKeys(key, newX, newY)) {
             case -2:
                 moveWindow(newX, newY);
-                /* nobreak */
+                /* fall-through */
             case -1:
                 endMoveSize();
                 break;
@@ -734,15 +712,13 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
                 setCurrentGeometryOuter(YRect(newX, newY, newWidth, newHeight));
                 drawMoveSizeFX(x(), y(), width(), height());
 
-#ifndef LITE
                 statusMoveSize->setStatus(this);
-#endif
                 break;
             case -2:
                 drawMoveSizeFX(x(), y(), width(), height());
                 setCurrentGeometryOuter(YRect(newX, newY, newWidth, newHeight));
                 drawMoveSizeFX(x(), y(), width(), height());
-                /* nobreak */
+                /* fall-through */
 
             case -1:
                 endMoveSize();
@@ -762,9 +738,9 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
             } else if (IS_WMKEY(k, vm, gKeyWinPrev)) {
                 wmPrevWindow();
             } else if (IS_WMKEY(k, vm, gKeyWinMaximizeVert)) {
-                wmMaximizeVert();
+                if (canMaximize()) wmMaximizeVert();
             } else if (IS_WMKEY(k, vm, gKeyWinMaximizeHoriz)) {
-                wmMaximizeHorz();
+                if (canMaximize()) wmMaximizeHorz();
             } else if (IS_WMKEY(k, vm, gKeyWinRaise)) {
                 if (canRaise()) wmRaise();
             } else if (IS_WMKEY(k, vm, gKeyWinOccupyAll)) {
@@ -881,81 +857,83 @@ bool YFrameWindow::canSize(bool horiz, bool vert) {
     return true;
 }
 
-bool YFrameWindow::canMove() {
-    if (!(frameFunctions() & ffMove))
-        return false;
-    return true;
+bool YFrameWindow::canMove() const {
+    return hasbit(frameFunctions(), ffMove);
 }
 
-#ifdef WMSPEC_HINTS
 void YFrameWindow::startMoveSize(int x, int y,
                                  int direction)
 {
-    int sx[] = { -1, 0, 1, 1, 1, 0, -1, -1, 0 };
-    int sy[] = { -1, -1, -1, 0, 1, 1, 1, 0, 0 };
+    int sx[] = { -1, 0, 1, 1, 1, 0, -1, -1, };
+    int sy[] = { -1, -1, -1, 0, 1, 1, 1, 0, };
 
-    if (direction >= 0 && direction < (int)(sizeof(sx)/sizeof(sx[0]))) {
-        MSG(("move size %d %d %d", x, y, direction));
-        if (direction == _NET_WM_MOVERESIZE_MOVE) {
-            x -= this->x();
-            y -= this->y();
-        }
-        startMoveSize((direction == _NET_WM_MOVERESIZE_MOVE) ? 1 : 0,
-                      true, sx[direction], sy[direction], x, y);
+    if (inrange(direction, 0, int ACOUNT(sx) - 1)) {
+        MSG(("move size %d %d direction %d", x, y, direction));
+        startMoveSize(false, true, sx[direction], sy[direction], x, y);
+    }
+    else if (direction == _NET_WM_MOVERESIZE_MOVE) {
+        MSG(("move size %d %d move by mouse", x, y));
+        startMoveSize(true, true, 0, 0, x - this->x(), y - this->y());
+    }
+    else if (direction == _NET_WM_MOVERESIZE_SIZE_KEYBOARD) {
+        MSG(("move size %d %d size by keyboard", x, y));
+        startMoveSize(false, false, 0, 0, x - this->x(), y - this->y());
+    }
+    else if (direction == _NET_WM_MOVERESIZE_MOVE_KEYBOARD) {
+        MSG(("move size %d %d move by keyboard", x, y));
+        startMoveSize(true, false, 0, 0, x - this->x(), y - this->y());
+    }
+    else if (direction == _NET_WM_MOVERESIZE_CANCEL) {
     } else
         warn(_("Unknown direction in move/resize request: %d"), direction);
 }
-#endif
 
-void YFrameWindow::startMoveSize(int doMove, int byMouse,
+void YFrameWindow::startMoveSize(bool doMove, bool byMouse,
                                  int sideX, int sideY,
                                  int mouseXroot, int mouseYroot) {
     Cursor grabPointer = None;
 
-    sizeByMouse = byMouse;
     grabX = sideX;
     grabY = sideY;
     origX = x();
     origY = y();
     origW = width();
     origH = height();
+    buttonDownX = 0;
+    buttonDownY = 0;
 
     manager->setWorkAreaMoveWindows(true);
     if (doMove && grabX == 0 && grabY == 0) {
         buttonDownX = mouseXroot;
         buttonDownY = mouseYroot;
 
-#ifdef CONFIG_GUIEVENTS
         wmapp->signalGuiEvent(geWindowMoved);
-#endif
-        grabPointer = YXApplication::movePointer.handle();
+        grabPointer = YXApplication::movePointer;
     } else if (!doMove) {
-#ifdef CONFIG_GUIEVENTS
         wmapp->signalGuiEvent(geWindowSized);
-#endif
 
         if (grabY == -1) {
             if (grabX == -1)
-                grabPointer = YWMApp::sizeTopLeftPointer.handle();
+                grabPointer = YWMApp::sizeTopLeftPointer;
             else if (grabX == 1)
-                grabPointer = YWMApp::sizeTopRightPointer.handle();
+                grabPointer = YWMApp::sizeTopRightPointer;
             else
-                grabPointer = YWMApp::sizeTopPointer.handle();
+                grabPointer = YWMApp::sizeTopPointer;
         } else if (grabY == 1) {
             if (grabX == -1)
-                grabPointer = YWMApp::sizeBottomLeftPointer.handle();
+                grabPointer = YWMApp::sizeBottomLeftPointer;
             else if (grabX == 1)
-                grabPointer = YWMApp::sizeBottomRightPointer.handle();
+                grabPointer = YWMApp::sizeBottomRightPointer;
             else
-                grabPointer = YWMApp::sizeBottomPointer.handle();
+                grabPointer = YWMApp::sizeBottomPointer;
         } else {
             if (grabX == -1)
-                grabPointer = YWMApp::sizeLeftPointer.handle();
+                grabPointer = YWMApp::sizeLeftPointer;
             else if (grabX == 1)
-                grabPointer = YWMApp::sizeRightPointer.handle();
+                grabPointer = YWMApp::sizeRightPointer;
             else
-                grabPointer = YXApplication::leftPointer.handle();
-            
+                grabPointer = YXApplication::leftPointer;
+
         }
 
         if (grabX == 1)
@@ -969,24 +947,21 @@ void YFrameWindow::startMoveSize(int doMove, int byMouse,
             buttonDownY = mouseYroot - y();
     }
 
-    XSync(xapp->display(), False);
     if (!xapp->grabEvents(this,
                           grabPointer,
                           ButtonPressMask |
                           ButtonReleaseMask |
                           PointerMotionMask))
     {
+        movingWindow = false;
+        sizingWindow = false;
         return ;
     }
 
-    if (doMove)
-        movingWindow = 1;
-    else
-        sizingWindow = 1;
+    movingWindow = doMove;
+    sizingWindow = !doMove;
 
-#ifndef LITE
     statusMoveSize->begin(this);
-#endif
 
     drawMoveSizeFX(x(), y(), width(), height());
 
@@ -1001,29 +976,25 @@ void YFrameWindow::startMoveSize(int doMove, int byMouse,
 
 void YFrameWindow::endMoveSize() {
     xapp->releaseEvents();
-#ifndef LITE
     statusMoveSize->end();
-#endif
 
     if ((movingWindow && opaqueMove) ||
         (sizingWindow && opaqueResize))
         drawMoveSizeFX(x(), y(), width(), height());
 
-    movingWindow = 0;
-    sizingWindow = 0;
+    movingWindow = false;
+    sizingWindow = false;
 
     manager->setWorkAreaMoveWindows(false);
 
-#ifdef CONFIG_TASKBAR
-    if (taskBar && taskBar->workspacesPane()) {
-        taskBar->workspacesPane()->repaint();
+    if (taskBar) {
+        taskBar->workspacesRepaint();
     }
-#endif
 }
 
 void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion) {
     if ((down.button == 3) && canMove()) {
-        startMoveSize(1, 1,
+        startMoveSize(true, true,
                       0, 0,
                       down.x, down.y);
         handleDrag(down, motion);
@@ -1032,23 +1003,23 @@ void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
         grabY = 0;
 
         if (down.x < int(borderX())) grabX = -1;
-        else if (width() - down.x <= borderX()) grabX = 1;
+        else if ((int) width() - down.x <= borderX()) grabX = 1;
 
         if (down.y < int(borderY())) grabY = -1;
-        else if (height() - down.y <= borderY()) grabY = 1;
+        else if ((int) height() - down.y <= borderY()) grabY = 1;
 
         if (grabY != 0 && grabX == 0) {
             if (down.x < int(wsCornerX)) grabX = -1;
-            else if ((int)width() - down.x <= wsCornerX) grabX = 1;
+            else if ((int)width() - down.x <= (int) wsCornerX) grabX = 1;
         }
 
         if (grabX != 0 && grabY == 0) {
             if (down.y < int(wsCornerY)) grabY = -1;
-            else if ((int)height() - down.y <= wsCornerY) grabY = 1;
+            else if ((int)height() - down.y <= (int) wsCornerY) grabY = 1;
         }
 
         if (grabX != 0 || grabY != 0) {
-            startMoveSize(0, 1,
+            startMoveSize(false, true,
                           grabX, grabY,
                           down.x_root, down.y_root);
 
@@ -1078,15 +1049,7 @@ void YFrameWindow::moveWindow(int newX, int newY) {
     if (opaqueMove)
         drawMoveSizeFX(x(), y(), width(), height());
 
-#ifndef LITE
     statusMoveSize->setStatus(this);
-#endif
-}
-
-void YFrameWindow::handleDrag(const XButtonEvent &/*down*/, const XMotionEvent &/*motion*/) {
-}
-
-void YFrameWindow::handleEndDrag(const XButtonEvent &/*down*/, const XButtonEvent &/*up*/) {
 }
 
 void YFrameWindow::handleButton(const XButtonEvent &button) {
@@ -1116,9 +1079,7 @@ void YFrameWindow::handleMotion(const XMotionEvent &motion) {
         setCurrentGeometryOuter(YRect(newX, newY, newWidth, newHeight));
         drawMoveSizeFX(x(), y(), width(), height());
 
-#ifndef LITE
         statusMoveSize->setStatus(this);
-#endif
         return ;
     } else if (movingWindow) {
         int newX = x();
@@ -1131,3 +1092,5 @@ void YFrameWindow::handleMotion(const XMotionEvent &motion) {
     YWindow::handleMotion(motion);
 }
 
+
+// vim: set sw=4 ts=4 et:
